param($installPath, $toolsPath, $package, $project)

##
## Inserts a new or updates an existing dependentAssembly element for a specified assembly
##
Function AddOrUpdateBindingRedirect([System.IO.FileInfo] $file, [System.Xml.XmlElement[]] $assemblyConfigs, [System.Xml.XmlDocument] $config)
{
    $name = [System.IO.Path]::GetFileNameWithoutExtension($file)
    $assemblyName = [System.Reflection.AssemblyName]::GetAssemblyName($file)

    $assemblyConfig =  $assemblyConfigs | ? { $_.assemblyIdentity.Name -Eq $name } 

    if ($assemblyConfig -Eq $null) 
    { 
        #there is no existing binding configuration for the assembly, we need to create a new config element for it
        Write-Host "Adding binding redirect for $name".

        $matches = $regex.Matches($assemblyName.FullName)
        if ($matches.Count -gt 0)
        {
	        $publicKeyToken = $matches[0].Groups["publicKeyToken"].Value
	        $culture = $matches[0].Groups["culture"].Value
        }
        else 
        {
            Write-Host "Unable to figure out culture and publicKeyToken for $name"
	        $publicKeyToken = "null"
	        $culture = "neutral"
        }
    
        $assemblyIdentity = $config.CreateElement("assemblyIdentity", $ns)
        $assemblyIdentity.SetAttribute("name", $name)
        if (![String]::IsNullOrEmpty($publicKeyToken))
        {
	        $assemblyIdentity.SetAttribute("publicKeyToken", $publicKeyToken)
        }
        if (![String]::IsNullOrEmpty($culture))
        {
	        $assemblyIdentity.SetAttribute("culture", $culture)
        }
        
        $bindingRedirect = $config.CreateElement("bindingRedirect", $ns)
        $bindingRedirect.SetAttribute("oldVersion", "")
        $bindingRedirect.SetAttribute("newVersion", "")
        
        $assemblyConfig = $config.CreateElement("dependentAssembly", $ns)
        $assemblyConfig.AppendChild($assemblyIdentity) | Out-Null
        $assemblyConfig.AppendChild($bindingRedirect) | Out-Null

        #locate the assemblyBinding element and append the newly created dependentAssembly element
        $assemblyBinding = $config.configuration.runtime.ChildNodes | where {$_.Name -eq "assemblyBinding"}
        $assemblyBinding.AppendChild($assemblyConfig) | Out-Null
    } 
    else 
    {
        Write-Host "Updating binding redirect for $name"
    }

    $assemblyConfig.bindingRedirect.oldVersion = "0.0.0.0-" + $assemblyName.Version
    $assemblyConfig.bindingRedirect.newVersion = $assemblyName.Version.ToString()
}

Function GetOrCreateXmlElement([System.Xml.XmlElement]$parent, $elementName, $ns, $document)
{
    $child = $parent.$($elementName)
    if ($child -eq $null) 
    {
        $child = $document.CreateElement($elementName, $ns)
        $parent.AppendChild($child) | Out-Null
    }
    $child
}


[regex]$regex = '[\w\.]+,\sVersion=[\d\.]+,\sCulture=(?<culture>[\w-]+),\sPublicKeyToken=(?<publicKeyToken>\w+)'
$ns = "urn:schemas-microsoft-com:asm.v1"
$libPath = join-path $installPath "lib\net461"
$projectFile = Get-Item $project.FullName

#locate the project configuration file
$webConfigPath = join-path $projectFile.Directory.FullName "web.config"
$appConfigPath = join-path $projectFile.Directory.FullName "app.config"
if (Test-Path $webConfigPath) 
{
    $configPath = $webConfigPath
}
elseif (Test-Path $appConfigPath)
{
    $configPath = $appConfigPath
}
else 
{
    Write-Host "Unable to find a configuration file, binding redirect not configured."
    return
}

#load the configuration file for the project
$config = New-Object xml
$config.psbase.PreserveWhitespace = $true
$config.Load($configPath)

# assume that we have the configuration element and make sure we have all the other parents of the AssemblyIdentity element.
$configElement = $config.configuration
$runtimeElement = GetOrCreateXmlElement $configElement "runtime" $null $config
$assemblyBindingElement = GetOrCreateXmlElement $runtimeElement "assemblyBinding" $ns $config

if ($assemblyBindingElement.length -gt 1)
{
    for ($i=1; $i -lt $assemblyBindingElement.length; $i++) 
    {
        $assemblyBindingElement[0].InnerXml +=  $assemblyBindingElement[$i].InnerXml
        $runtimeElement.RemoveChild($assemblyBindingElement[$i])
    }
    $config.Save($configPath)
}

else 
{
    $assemblyBindingElement = @($assemblyBindingElement)
}

$assemblyConfigs = $assemblyBindingElement[0].ChildNodes | where {$_.GetType().Name -eq "XmlElement"}

#add/update binding redirects for assemblies in the current package
get-childItem "$libPath\*.dll" | % { AddOrUpdateBindingRedirect $_  $assemblyConfigs $config }

$config.Save($configPath)
# SIG # Begin signature block
# MIIVugYJKoZIhvcNAQcCoIIVqzCCFacCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAgeVbA6x5m0vrR
# kBmJ8IIwefePqTT+7TzdnltNmRxIUqCCEfYwggVvMIIEV6ADAgECAhBI/JO0YFWU
# jTanyYqJ1pQWMA0GCSqGSIb3DQEBDAUAMHsxCzAJBgNVBAYTAkdCMRswGQYDVQQI
# DBJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcMB1NhbGZvcmQxGjAYBgNVBAoM
# EUNvbW9kbyBDQSBMaW1pdGVkMSEwHwYDVQQDDBhBQUEgQ2VydGlmaWNhdGUgU2Vy
# dmljZXMwHhcNMjEwNTI1MDAwMDAwWhcNMjgxMjMxMjM1OTU5WjBWMQswCQYDVQQG
# EwJHQjEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMS0wKwYDVQQDEyRTZWN0aWdv
# IFB1YmxpYyBDb2RlIFNpZ25pbmcgUm9vdCBSNDYwggIiMA0GCSqGSIb3DQEBAQUA
# A4ICDwAwggIKAoICAQCN55QSIgQkdC7/FiMCkoq2rjaFrEfUI5ErPtx94jGgUW+s
# hJHjUoq14pbe0IdjJImK/+8Skzt9u7aKvb0Ffyeba2XTpQxpsbxJOZrxbW6q5KCD
# J9qaDStQ6Utbs7hkNqR+Sj2pcaths3OzPAsM79szV+W+NDfjlxtd/R8SPYIDdub7
# P2bSlDFp+m2zNKzBenjcklDyZMeqLQSrw2rq4C+np9xu1+j/2iGrQL+57g2extme
# me/G3h+pDHazJyCh1rr9gOcB0u/rgimVcI3/uxXP/tEPNqIuTzKQdEZrRzUTdwUz
# T2MuuC3hv2WnBGsY2HH6zAjybYmZELGt2z4s5KoYsMYHAXVn3m3pY2MeNn9pib6q
# RT5uWl+PoVvLnTCGMOgDs0DGDQ84zWeoU4j6uDBl+m/H5x2xg3RpPqzEaDux5mcz
# mrYI4IAFSEDu9oJkRqj1c7AGlfJsZZ+/VVscnFcax3hGfHCqlBuCF6yH6bbJDoEc
# QNYWFyn8XJwYK+pF9e+91WdPKF4F7pBMeufG9ND8+s0+MkYTIDaKBOq3qgdGnA2T
# OglmmVhcKaO5DKYwODzQRjY1fJy67sPV+Qp2+n4FG0DKkjXp1XrRtX8ArqmQqsV/
# AZwQsRb8zG4Y3G9i/qZQp7h7uJ0VP/4gDHXIIloTlRmQAOka1cKG8eOO7F/05QID
# AQABo4IBEjCCAQ4wHwYDVR0jBBgwFoAUoBEKIz6W8Qfs4q8p74Klf9AwpLQwHQYD
# VR0OBBYEFDLrkpr/NZZILyhAQnAgNpFcF4XmMA4GA1UdDwEB/wQEAwIBhjAPBgNV
# HRMBAf8EBTADAQH/MBMGA1UdJQQMMAoGCCsGAQUFBwMDMBsGA1UdIAQUMBIwBgYE
# VR0gADAIBgZngQwBBAEwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybC5jb21v
# ZG9jYS5jb20vQUFBQ2VydGlmaWNhdGVTZXJ2aWNlcy5jcmwwNAYIKwYBBQUHAQEE
# KDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5jb21vZG9jYS5jb20wDQYJKoZI
# hvcNAQEMBQADggEBABK/oe+LdJqYRLhpRrWrJAoMpIpnuDqBv0WKfVIHqI0fTiGF
# OaNrXi0ghr8QuK55O1PNtPvYRL4G2VxjZ9RAFodEhnIq1jIV9RKDwvnhXRFAZ/ZC
# J3LFI+ICOBpMIOLbAffNRk8monxmwFE2tokCVMf8WPtsAO7+mKYulaEMUykfb9gZ
# pk+e96wJ6l2CxouvgKe9gUhShDHaMuwV5KZMPWw5c9QLhTkg4IUaaOGnSDip0TYl
# d8GNGRbFiExmfS9jzpjoad+sPKhdnckcW67Y8y90z7h+9teDnRGWYpquRRPaf9xH
# +9/DUp/mBlXpnYzyOmJRvOwkDynUWICE5EV7WtgwggYaMIIEAqADAgECAhBiHW0M
# UgGeO5B5FSCJIRwKMA0GCSqGSIb3DQEBDAUAMFYxCzAJBgNVBAYTAkdCMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxLTArBgNVBAMTJFNlY3RpZ28gUHVibGljIENv
# ZGUgU2lnbmluZyBSb290IFI0NjAeFw0yMTAzMjIwMDAwMDBaFw0zNjAzMjEyMzU5
# NTlaMFQxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxKzAp
# BgNVBAMTIlNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBSMzYwggGiMA0G
# CSqGSIb3DQEBAQUAA4IBjwAwggGKAoIBgQCbK51T+jU/jmAGQ2rAz/V/9shTUxjI
# ztNsfvxYB5UXeWUzCxEeAEZGbEN4QMgCsJLZUKhWThj/yPqy0iSZhXkZ6Pg2A2NV
# DgFigOMYzB2OKhdqfWGVoYW3haT29PSTahYkwmMv0b/83nbeECbiMXhSOtbam+/3
# 6F09fy1tsB8je/RV0mIk8XL/tfCK6cPuYHE215wzrK0h1SWHTxPbPuYkRdkP05Zw
# mRmTnAO5/arnY83jeNzhP06ShdnRqtZlV59+8yv+KIhE5ILMqgOZYAENHNX9SJDm
# +qxp4VqpB3MV/h53yl41aHU5pledi9lCBbH9JeIkNFICiVHNkRmq4TpxtwfvjsUe
# dyz8rNyfQJy/aOs5b4s+ac7IH60B+Ja7TVM+EKv1WuTGwcLmoU3FpOFMbmPj8pz4
# 4MPZ1f9+YEQIQty/NQd/2yGgW+ufflcZ/ZE9o1M7a5Jnqf2i2/uMSWymR8r2oQBM
# dlyh2n5HirY4jKnFH/9gRvd+QOfdRrJZb1sCAwEAAaOCAWQwggFgMB8GA1UdIwQY
# MBaAFDLrkpr/NZZILyhAQnAgNpFcF4XmMB0GA1UdDgQWBBQPKssghyi47G9IritU
# pimqF6TNDDAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBADATBgNV
# HSUEDDAKBggrBgEFBQcDAzAbBgNVHSAEFDASMAYGBFUdIAAwCAYGZ4EMAQQBMEsG
# A1UdHwREMEIwQKA+oDyGOmh0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1B1
# YmxpY0NvZGVTaWduaW5nUm9vdFI0Ni5jcmwwewYIKwYBBQUHAQEEbzBtMEYGCCsG
# AQUFBzAChjpodHRwOi8vY3J0LnNlY3RpZ28uY29tL1NlY3RpZ29QdWJsaWNDb2Rl
# U2lnbmluZ1Jvb3RSNDYucDdjMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0
# aWdvLmNvbTANBgkqhkiG9w0BAQwFAAOCAgEABv+C4XdjNm57oRUgmxP/BP6YdURh
# w1aVcdGRP4Wh60BAscjW4HL9hcpkOTz5jUug2oeunbYAowbFC2AKK+cMcXIBD0Zd
# OaWTsyNyBBsMLHqafvIhrCymlaS98+QpoBCyKppP0OcxYEdU0hpsaqBBIZOtBajj
# cw5+w/KeFvPYfLF/ldYpmlG+vd0xqlqd099iChnyIMvY5HexjO2AmtsbpVn0OhNc
# WbWDRF/3sBp6fWXhz7DcML4iTAWS+MVXeNLj1lJziVKEoroGs9Mlizg0bUMbOalO
# hOfCipnx8CaLZeVme5yELg09Jlo8BMe80jO37PU8ejfkP9/uPak7VLwELKxAMcJs
# zkyeiaerlphwoKx1uHRzNyE6bxuSKcutisqmKL5OTunAvtONEoteSiabkPVSZ2z7
# 6mKnzAfZxCl/3dq3dUNw4rg3sTCggkHSRqTqlLMS7gjrhTqBmzu1L90Y1KWN/Y5J
# KdGvspbOrTfOXyXvmPL6E52z1NZJ6ctuMFBQZH3pwWvqURR8AgQdULUvrxjUYbHH
# j95Ejza63zdrEcxWLDX6xWls/GDnVNueKjWUH3fTv1Y8Wdho698YADR7TNx8X8z2
# Bev6SivBBOHY+uqiirZtg0y9ShQoPzmCcn63Syatatvx157YK9hlcPmVoa1oDE5/
# L9Uo2bC5a4CH2RwwggZhMIIEyaADAgECAhBcAaBVWnoCn7o9oyyplGP8MA0GCSqG
# SIb3DQEBDAUAMFQxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0
# ZWQxKzApBgNVBAMTIlNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBSMzYw
# HhcNMjIwNTE4MDAwMDAwWhcNMjUwNTE3MjM1OTU5WjBVMQswCQYDVQQGEwJTRTEY
# MBYGA1UECAwPU3RvY2tob2xtcyBsw6RuMRUwEwYDVQQKDAxFcGlzZXJ2ZXIgQUIx
# FTATBgNVBAMMDEVwaXNlcnZlciBBQjCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCC
# AgoCggIBAKiNgBKeBci/YSf1oGuEPAaMUNnPIWPB3ZuyQadDOfada0CZS/AXlzpK
# UjqkyTgjudSR0qtneowY7HKvSDYe3ZHiNbeGdsbARtyVq40X+mpxd8PL55NwA0F7
# gdV0xN3+M5y8CDlmGzOOTQo1GXdfFQ5GPt5aB9f3qd2wk2f+wPTsaRLxkUddMTKg
# 5bbr94nSo4ZFaUw2shzcmegYeNJGtgmaOyOCgcM5WZmslF9Q5TvFa57E1odbEMe3
# iukZ+ORprPDs0wVnFPj5f1HwH+TmoVo9wOv95mcglWU1J3P1RDEXYn8sRtLtrbTt
# Ox6rlPHiRaQ9heRmckHwMc7Pc5l6MiU0VK8DabXonESf74trNlnL4bSCQthU1Vtf
# V2CyRVYmupLhZUYuMBzOp4G1VDwrGxQR/I0z23TXSz9v0BzcJKsIdU7F1ZJsCejh
# J7NgjhraAkxk4RkrNxnTEfbftuIRAW59j+ITrecWq0+AXGyK/JRrOzQjVOJzaq7/
# qrh2tpsP+8QGNTtEzsGw5S+oktjneTczJREksSBMXLU20HgxRrsJOIhUseU/Bb38
# oXpXMbNcAEucqF6RgXt5VEm+t8OM0/bZYLZVXLmw7FM6gKdIZrwoPD0ksEPBb1GB
# u59w1j0Tnr2z+rCrfk8mKWMQcwziyh8W72ljZ3WIY5MnanhWmDlBAgMBAAGjggGs
# MIIBqDAfBgNVHSMEGDAWgBQPKssghyi47G9IritUpimqF6TNDDAdBgNVHQ4EFgQU
# 3diA9r3UvSgphZLmtLNffpcnGsYwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQC
# MAAwEwYDVR0lBAwwCgYIKwYBBQUHAwMwSgYDVR0gBEMwQTA1BgwrBgEEAbIxAQIB
# AwIwJTAjBggrBgEFBQcCARYXaHR0cHM6Ly9zZWN0aWdvLmNvbS9DUFMwCAYGZ4EM
# AQQBMEkGA1UdHwRCMEAwPqA8oDqGOGh0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2Vj
# dGlnb1B1YmxpY0NvZGVTaWduaW5nQ0FSMzYuY3JsMHkGCCsGAQUFBwEBBG0wazBE
# BggrBgEFBQcwAoY4aHR0cDovL2NydC5zZWN0aWdvLmNvbS9TZWN0aWdvUHVibGlj
# Q29kZVNpZ25pbmdDQVIzNi5jcnQwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLnNl
# Y3RpZ28uY29tMCEGA1UdEQQaMBiBFnN1cHBvcnRAb3B0aW1pemVseS5jb20wDQYJ
# KoZIhvcNAQEMBQADggGBAEiZYZ7G62amIIaf//0rswdatkIdmygqPraFkZvyOeoi
# LOh3sty6WApVMkhfvAtTQiEmdO0Q9HQYKy1GQEDtTrkPx9uZP1GGqcMcD/6sC5Tc
# 26U1uY4YAzm0svvK9ujf7wRxaO9py7x45d5obc78IYaH3ymH3hqS+SOD9iVRi4oT
# yNhS1zTuBVLoYvMnHnHmfPhoG7bQL2BwbkPlcz8hZBatCGgJu9HGlSgmk574ZKMD
# fyUIzZ/4KHklSY18DZtWfbKsSVOkaTJ0e0vdZ2R+vzq7BIBZS7YKRrFg0cGZDS73
# CZCE+hBC7xctmWnprttHuFLI+db4EXy/A5p4w/z2f6/WHk9oNYcW08stSSWqVINu
# JzPdCFS2siMI1G1d02c9NHsLAK2ne7N2qnEhZu5nQHyk8R2tN8XjlPIfJ2SPJWR5
# TzObBLsFqeCsbYB7b+GfrsK9E4jtu3HXT4IDaSPxRXrOVr+JzCu/LFyat7vSCjKF
# BDfv/wRuh1U5oiJMUvk17DGCAxowggMWAgEBMGgwVDELMAkGA1UEBhMCR0IxGDAW
# BgNVBAoTD1NlY3RpZ28gTGltaXRlZDErMCkGA1UEAxMiU2VjdGlnbyBQdWJsaWMg
# Q29kZSBTaWduaW5nIENBIFIzNgIQXAGgVVp6Ap+6PaMsqZRj/DANBglghkgBZQME
# AgEFAKCBhDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEM
# BgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqG
# SIb3DQEJBDEiBCA6hQPeoVWDN0Pz5YyqQNtnTuVKLeaKAjzLN8N6T15qWzANBgkq
# hkiG9w0BAQEFAASCAgAZzr8RoQ2a8rMJUrPYaJtetaKFw/ymuJ3PvztbmKo5fy/s
# x73YfaSG0d+Oj47/UBwtCiJB6FEULOQuTJXyb5suRHow1Vecba8uXv998K9TQoqR
# ubeuXfGwxzFGmPI5wmOEdORZ1X5g+hlzixS9rYcY2Mg1lVttQP4hvFXnDQffggT4
# 3itCm5VLZfMAuXHkJzU2p7kaCwi2ZfzgAxOBLpYLseAPyix1b5E1vyqbiXn2bFhh
# 4xrRlNjHRpw+8Rr2ImhAWXa6JnzqZLYwyW5tjn3Va25eNG9FBJmdBYTCvT9bg4d8
# pXe46cLEc/DjPwXxFi5nVd8URxiUv2u7x3GpZtpvZD8KnYTialEGmwDR6arDB1/U
# ffZtZnjQm1d1grpo+41n5FeHJY4+UjhRmi7o9r2Vw6bFDzJSqP9w0xamyITSyqvX
# vnhuXoyZAe5MhsE50p/9gdiamhRxNFpVw0imORw+20n56Nvdew8CbC1Z0wKPLjbI
# /oASF8Y2+1ouB3TJTsw4X2tNd02HmHih51Lgw2RNfMwVjsjWlsJeKL4UKU/rya01
# SrLzwvLvWuxj3bO8NRPSTRuJlWw27xVKEWg2Kc+i3B60yxP0cBm1ZLRAZHoHv4T8
# u1cwOJu6qz2ke8MNEAVfzsLBT8YmCRIxCuEiPfOEY5EfJ1UMsNwtLho5LaiV+g==
# SIG # End signature block
